import typing

import databases
from aioworkers.core.base import AbstractConnector
from aioworkers.core.config import Config
from databases.core import Connection, Transaction
from sqlalchemy.sql import ClauseElement


class Database(AbstractConnector):
    """
    Use Wrapper (Decorator) pattern to wrap databases.Database and add it behavior to entity.
    """

    def __init__(self, config=None, *, context=None, loop=None):
        super().__init__(config, context=context, loop=loop)
        self._db: databases.Database = None
        self._dsn: str = None

    async def init(self):
        await super().init()
        self._db = databases.Database(self._dsn)

    def set_config(self, config: Config) -> None:
        cfg = config.new_parent(logger="aioworkers_databases")
        self._dsn = cfg.get("dsn")
        super().set_config(cfg)

    async def connect(self):
        await self._db.connect()

    async def disconnect(self):
        await self._db.disconnect()

    async def execute(
        self, query: typing.Union[ClauseElement, str], values: dict = None
    ) -> typing.Any:
        return await self._db.execute(query, values)

    async def execute_many(
        self, query: typing.Union[ClauseElement, str], values: list
    ) -> None:
        return await self._db.execute_many(query, values)

    async def fetch_all(
        self, query: typing.Union[ClauseElement, str], values: dict = None
    ) -> typing.List[typing.Mapping]:
        return await self._db.fetch_all(query, values)

    async def fetch_one(
        self, query: typing.Union[ClauseElement, str], values: dict = None
    ) -> typing.Optional[typing.Mapping]:
        return await self._db.fetch_one(query, values)

    async def fetch_val(
        self,
        query: typing.Union[ClauseElement, str],
        values: dict = None,
        column: typing.Any = 0,
    ) -> typing.Any:
        return await self._db.fetch_val(query, values, column)

    def transaction(
        self, *, force_rollback: bool = False, **kwargs: typing.Any
    ) -> "Transaction":
        return self._db.transaction(force_rollback=force_rollback, **kwargs)

    def iterate(
        self, query: typing.Union[ClauseElement, str], values: dict = None
    ) -> typing.AsyncGenerator[typing.Mapping, None]:
        return self._db.iterate(query, values)

    def connection(self) -> "Connection":
        return self._db.connection()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aioworkers_databases']

package_data = \
{'': ['*']}

install_requires = \
['aioworkers>=0.18,<0.19', 'databases>=0.4.0,<0.5.0']

setup_kwargs = {
    'name': 'aioworkers-databases',
    'version': '0.1.1',
    'description': 'Databases plugin for aioworkers',
    'long_description': "# aioworkers-databases\n\n[![GitHub Workflow Status](https://img.shields.io/github/workflow/status/aioworkers/aioworkers-databases/CI)](https://github.com/aioworkers/aioworkers-databases/actions?query=workflow%3ACI)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/aioworkers-databases)](https://pypi.org/project/aioworkers-databases)\n[![PyPI](https://img.shields.io/pypi/v/aioworkers-databases)](https://pypi.org/project/aioworkers-databases)\n\naioworkers plugin for [databases](https://github.com/encode/databases).\n\n## Usage\n\nAdd your database config to aioworkers `config.yaml`:\n\n```yaml\ndb:\n  cls: aioworkers_databases.database.Database\n  dsn: sqlite:///db.sqlite\nlogging:\n  version: 1\n  disable_existing_loggers: false\n  root:\n    level: ERROR\n    handlers: [console]\n  handlers:\n    console:\n      level: DEBUG\n      class: logging.StreamHandler\n  loggers:\n    aioworkers_databases:\n      level: ERROR\n      handlers: [console]\n      propagate: true\n```\n\nRun `aioworkers`:\n\n```shell\naioworkers -c config.yaml -i        \n```\n\nCreate `Context` for this config and use your db via context:\n\n```python\nawait context.db.execute('CREATE TABLE some_table (id INT);')\n```\n\n## Development\n\nInstall requirements:\n\n```shell\npoetry install\n```\n\nRun tests:\n\n```shell\npytest\n```",
    'author': 'Alexander Bogushov',
    'author_email': 'abogushov@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/aioworkers/aioworkers-databases',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0',
}


setup(**setup_kwargs)
